using System;
using System.Windows.Forms;
using System.Collections;

namespace GEN
{
	class GENConf
	{
		private int _numNodes;
		private char[,] _SA; 
		//variables for random values
		private Sequence[] _sequenceList;
		private int _numDes;
		private int _SA_Width;
		private int _SA_Height;
		private int _Speed;
		private int _PauseTime;
		
		/// <summary>
		/// Default constructor
		/// </summary>
		public GENConf()
		{
			_numNodes = 4;//default node number
			_SA_Width = 200;
			_SA_Height = 200;
			_SA = new char[_SA_Width,_SA_Height];
			_Speed = int.MaxValue;
			_PauseTime = int.MaxValue;
			_numDes = 10;
			createSequence();
		}

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="p_num_of_nodes">number of nodes</param>
		/// <param name="p_time_interval">time interval(pause time)</param>
		/// <param name="p_time_duration">time duration(whole period of time)</param>
		/// <param name="row_length">length of row</param>
		/// <param name="col_length">length of col</param>
		public GENConf(
			int p_numNodes, 
			int p_numDes,
			int p_SA_Width,
			int p_SA_Height,
			int p_Speed,
			int p_PauseTime)
		{
			_numNodes = p_numNodes;
			_SA_Width = p_SA_Width;
			_SA_Height = p_SA_Height;
			_Speed = p_Speed;
			_PauseTime = p_PauseTime;
			_numDes = p_numDes;
			createSequence();
			_SA = new char[_SA_Width,_SA_Height];

		}

		/// <summary>
		/// This method is used in constructors 
		/// to create Sequence instance
		/// </summary>
		/// <param name="n_list"> length of list</param>
		private void createSequence()
		{
			try
			{
				_sequenceList = new Sequence[_numDes];
			
				for(int i=0; i<_numDes; i++)
				{
					_sequenceList[i] = new Sequence();
				}
			}
			catch(Exception e)
			{
               MessageBox.Show("Exception in createSequence(); "+e.Message);
			}
		}


		/// <summary>
		/// Always generate a same seeds so that 
		/// the randomly generated (destination, speed, pause) 
		/// should be also same sequence whenever it runs.
		/// </summary>
		/// <returns>seed list</returns>
		public int[] generateSeeds(int p_seed)
		{
			int length = _numNodes;
			int[] seedList = new int[length];
			Random r = new Random(p_seed);
			for(int i=0; i<length; i++)
			{
				seedList[i] = r.Next();
			}
			return seedList;
		}

		private ArrayList getRandomDestionationList(int p_seed)
		{
			ArrayList DesList = new ArrayList();
			Random r = new Random(p_seed);
			Random r_row = new Random(r.Next());
			Random r_col = new Random(r.Next());
			try
			{
				for(int i = 0; i<_numDes; i++)
				{
                    Point point = new Point(r_row.Next(0, _SA_Height), 
						                    r_col.Next(0, _SA_Width));
					DesList.Add(point);
				}
			}
			catch(Exception e)
			{
                MessageBox.Show("Exception in getRandomDestionationList(): "+e.Message);
			}
			return DesList;
		}
        		
		private ArrayList getSpeedList(int p_seed)
		{
			ArrayList speedList = new ArrayList();
			try
			{
				if(_Speed == int.MaxValue)//Random Speed Selected
				{
					Random r_speed = new Random(p_seed);
					for(int i = 0; i<_numDes; i++)
					{
						speedList.Add(r_speed.Next(10, 640));
					}
				}
				else//specific speed is entered
				{
					for(int i =0 ; i<_numDes; i++)
					{
                        speedList.Add(_Speed);
					}
				}                
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getSpeedList(): "+e.Message);
			}
			return speedList;
		}

		private ArrayList getPauseTimeList(int p_seed)
		{
			ArrayList PauseTimeList = new ArrayList();
			try
			{
				if(_PauseTime == int.MaxValue)
				{
					Random r_pause = new Random(p_seed);
					for(int i = 0; i<_numDes; i++)
					{
                       PauseTimeList.Add(r_pause.Next(0, 100));
					}
				}
				else// Pause Time is specified.
				{
					for(int i = 0; i<_numDes; i++)
					{
						PauseTimeList.Add(_PauseTime);
					}
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getPauseTimeList(): "+e.Message);
			}
			return PauseTimeList;
		}

		public void generateSequenceForOneNode(int p_seed)
		{
			try
			{		
				ArrayList DesList = getRandomDestionationList(p_seed);				
				ArrayList PauseTimeList = getPauseTimeList(p_seed);				
			    ArrayList speedList= getSpeedList(p_seed);	
				
				for(int i = 0; i<_numDes; i++)
				{
					Point tempDes = (Point)DesList[i];
                    _sequenceList[i].destination.row = tempDes.row;
                    _sequenceList[i].destination.col = tempDes.col;
					_sequenceList[i].speed = int.Parse(speedList[i].ToString());
					_sequenceList[i].pause = int.Parse(PauseTimeList[i].ToString());
				}
			}
			catch(Exception e)
			{
               MessageBox.Show("Exception in generateSequenceForOneNode(): "+e.Message);
			}
		}

		public Sequence[] getSequenceList()
		{
			return _sequenceList;  
		}

	}//class
	
	public struct Point
	{
		public double row;
		public double col;
	
		public Point(double p_row, double p_col)
		{
			row = Convert.ToDouble(Math.Round(Convert.ToDecimal(p_row),2));
			col = Convert.ToDouble(Math.Round(Convert.ToDecimal(p_col),2));
		}

	}//end of struct

	struct Sequence
	{
		public int speed;
		public int pause;
		public Point destination;
 
	}//end of struct
}//end of namespace
