/*Author: Jin-Hee Cho 
* Project Name: Wireless Ad Hoc Messenger Using Pocket PCs
* Purpose: Thesis for Master Science in Computer Science 
* at Virginia Polytechnic State University
* Funded by Microsoft
* Date Updated: 7/29/2004
* GEN is the application to generate simulated network 
* topology based on the random waypoint model.
*/

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Xml;
using System.Text;
using System.IO;

namespace GEN
{
	/// <summary>
	/// Summary description for GENUI.
	/// </summary>
	public class GENUI : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Label label1;//Node ID
		private System.Windows.Forms.Label label2;//Node IP
		private System.Windows.Forms.Button btnAddNode;//button to add a node
		private System.Windows.Forms.TextBox txtbox_id;//textbox for ip input
		private System.Windows.Forms.ListView NodelistView;//listView
		public System.Windows.Forms.Panel panel1;//First page to add nodes
		private System.Windows.Forms.Button btnGENConf;
		private System.Windows.Forms.TextBox textBox_IP1;
		private System.Windows.Forms.TextBox textBox_IP2;
		private System.Windows.Forms.TextBox textBox_IP3;
		private System.Windows.Forms.TextBox textBox_IP4;
		private System.Windows.Forms.Button btnRmvNode;
		private System.Windows.Forms.Button btnReset;
		private System.Windows.Forms.MainMenu mainMenu1;
		private System.Windows.Forms.CheckBox ACM_CB;
		//On the panel_Variable
		private System.Windows.Forms.Panel panel_Variable;
		private System.Windows.Forms.Button Set_VarBtn;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.TextBox SA_Width_TB;
		private System.Windows.Forms.TextBox SA_Height_TB;
		private System.Windows.Forms.Label label4;
		private System.Windows.Forms.Label label5;
		private System.Windows.Forms.Label label6;
		private System.Windows.Forms.TextBox Power_Range_TB;
		private System.Windows.Forms.Button Back_Btn;
		private System.Windows.Forms.Panel panel_Speed;
		private System.Windows.Forms.Label label7;
		private System.Windows.Forms.Panel panel_Pause_Time;
		private System.Windows.Forms.Label label8;
		private System.Windows.Forms.RadioButton PT_30_RB;
		private System.Windows.Forms.RadioButton PT_60_RB;
		private System.Windows.Forms.RadioButton PT_120_RB;
		private System.Windows.Forms.RadioButton PT_240_RB;
		private System.Windows.Forms.RadioButton PT_480_RB;
		private System.Windows.Forms.RadioButton PT_960_RB;

		//variables to hold text box input values
		private string strID="";
		private string strIP="";
		private Hashtable id_ip_hash;
		private ArrayList all_sequences;
		private XmlTextWriter confWriter;
		private XmlTextWriter basicInfoWriter;
		private Hashtable neighbors_hash;
		private static int i = 0;
		private bool _isOnACM = false;
		private XmlDocument xmlDoc;
		private int _SA_Width = 0;
		private int _SA_Height = 0;
		private int _PowerRange = 0;
		private int _Speed = 0;
		private System.Windows.Forms.TextBox Seed_TB;
		private System.Windows.Forms.Label label9;
		private System.Windows.Forms.Button Exit_Btn;
		private System.Windows.Forms.RadioButton PT_15_RB;
		private System.Windows.Forms.RadioButton PT_0_RB;
		private System.Windows.Forms.RadioButton Speed_0_RB;
		private System.Windows.Forms.RadioButton Speed_2_RB;
		private System.Windows.Forms.RadioButton Speed_4_RB;
		private System.Windows.Forms.RadioButton Speed_6_RB;
		private System.Windows.Forms.RadioButton Speed_8_RB;
		private System.Windows.Forms.RadioButton Speed_10_RB;
		private System.Windows.Forms.RadioButton Speed_12_RB;
		private System.Windows.Forms.RadioButton Speed_80_RB;
		private int _PauseTime = 0;

		public GENUI()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();  			
			SetInitial_id_ip_hash();
			
			neighbors_hash = new Hashtable();
			xmlDoc = new XmlDocument();
			textBox_IP1.MaxLength = 3;
			textBox_IP2.MaxLength = 3;
			textBox_IP3.MaxLength = 3;
			textBox_IP4.MaxLength = 3;
			//
			// TODO: Add any constructor code after InitializeComponent call
			//
		}

		private void SetInitial_id_ip_hash()
		{
			try
			{
				id_ip_hash = new Hashtable();
				string id_A = "A";
				string ip_A = "192.168.0.25";
				string id_B = "B";
				string ip_B = "192.168.0.23";
				string id_C = "C";
				string ip_C = "192.168.0.24";
				string id_D = "D";
				string ip_D = "192.168.0.26";
				string id_E = "E";
				string ip_E = "192.168.0.27";
				string id_F = "F";
				string ip_F = "192.168.0.28";
				string id_G = "G";
				string ip_G = "192.168.0.29";

				id_ip_hash.Add(id_A, ip_A);
				id_ip_hash.Add(id_B, ip_B);
				id_ip_hash.Add(id_C, ip_C);
				id_ip_hash.Add(id_D, ip_D);
				id_ip_hash.Add(id_E, ip_E);	
				id_ip_hash.Add(id_F, ip_F);	
				id_ip_hash.Add(id_G, ip_G);				
			}
			catch(Exception e)
			{
                MessageBox.Show("Exception in SetInitial_id_ip_hash(): "+e.Message);
			}

		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			base.Dispose( disposing );
		}

		static void Main()
		{
			Application.Run(new GENUI());
		}
		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.label1 = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.btnAddNode = new System.Windows.Forms.Button();
			this.txtbox_id = new System.Windows.Forms.TextBox();
			this.NodelistView = new System.Windows.Forms.ListView();
			this.panel1 = new System.Windows.Forms.Panel();
			this.Exit_Btn = new System.Windows.Forms.Button();
			this.Set_VarBtn = new System.Windows.Forms.Button();
			this.ACM_CB = new System.Windows.Forms.CheckBox();
			this.btnReset = new System.Windows.Forms.Button();
			this.btnRmvNode = new System.Windows.Forms.Button();
			this.textBox_IP4 = new System.Windows.Forms.TextBox();
			this.textBox_IP3 = new System.Windows.Forms.TextBox();
			this.textBox_IP2 = new System.Windows.Forms.TextBox();
			this.textBox_IP1 = new System.Windows.Forms.TextBox();
			this.btnGENConf = new System.Windows.Forms.Button();
			this.mainMenu1 = new System.Windows.Forms.MainMenu();
			this.panel_Variable = new System.Windows.Forms.Panel();
			this.label9 = new System.Windows.Forms.Label();
			this.Seed_TB = new System.Windows.Forms.TextBox();
			this.panel_Speed = new System.Windows.Forms.Panel();
			this.Speed_80_RB = new System.Windows.Forms.RadioButton();
			this.Speed_12_RB = new System.Windows.Forms.RadioButton();
			this.Speed_10_RB = new System.Windows.Forms.RadioButton();
			this.Speed_8_RB = new System.Windows.Forms.RadioButton();
			this.Speed_6_RB = new System.Windows.Forms.RadioButton();
			this.Speed_4_RB = new System.Windows.Forms.RadioButton();
			this.Speed_2_RB = new System.Windows.Forms.RadioButton();
			this.Speed_0_RB = new System.Windows.Forms.RadioButton();
			this.label7 = new System.Windows.Forms.Label();
			this.panel_Pause_Time = new System.Windows.Forms.Panel();
			this.PT_30_RB = new System.Windows.Forms.RadioButton();
			this.label8 = new System.Windows.Forms.Label();
			this.PT_15_RB = new System.Windows.Forms.RadioButton();
			this.PT_60_RB = new System.Windows.Forms.RadioButton();
			this.PT_120_RB = new System.Windows.Forms.RadioButton();
			this.PT_240_RB = new System.Windows.Forms.RadioButton();
			this.PT_480_RB = new System.Windows.Forms.RadioButton();
			this.PT_960_RB = new System.Windows.Forms.RadioButton();
			this.PT_0_RB = new System.Windows.Forms.RadioButton();
			this.Back_Btn = new System.Windows.Forms.Button();
			this.Power_Range_TB = new System.Windows.Forms.TextBox();
			this.label6 = new System.Windows.Forms.Label();
			this.label5 = new System.Windows.Forms.Label();
			this.label4 = new System.Windows.Forms.Label();
			this.SA_Height_TB = new System.Windows.Forms.TextBox();
			this.SA_Width_TB = new System.Windows.Forms.TextBox();
			this.label3 = new System.Windows.Forms.Label();
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(8, 16);
			this.label1.Size = new System.Drawing.Size(56, 16);
			this.label1.Text = "Add ID :";
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(8, 40);
			this.label2.Size = new System.Drawing.Size(48, 16);
			this.label2.Text = "Add IP :";
			// 
			// btnAddNode
			// 
			this.btnAddNode.Location = new System.Drawing.Point(8, 72);
			this.btnAddNode.Size = new System.Drawing.Size(56, 24);
			this.btnAddNode.Text = "Add ";
			this.btnAddNode.Click += new System.EventHandler(this.btnAddNode_Click);
			// 
			// txtbox_id
			// 
			this.txtbox_id.Location = new System.Drawing.Point(72, 8);
			this.txtbox_id.Size = new System.Drawing.Size(160, 22);
			this.txtbox_id.Text = "";
			// 
			// NodelistView
			// 
			this.NodelistView.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
			this.NodelistView.Location = new System.Drawing.Point(8, 160);
			this.NodelistView.Size = new System.Drawing.Size(224, 104);
			// 
			// panel1
			// 
			this.panel1.BackColor = System.Drawing.Color.Gold;
			this.panel1.Controls.Add(this.Exit_Btn);
			this.panel1.Controls.Add(this.Set_VarBtn);
			this.panel1.Controls.Add(this.ACM_CB);
			this.panel1.Controls.Add(this.btnReset);
			this.panel1.Controls.Add(this.btnRmvNode);
			this.panel1.Controls.Add(this.textBox_IP4);
			this.panel1.Controls.Add(this.textBox_IP3);
			this.panel1.Controls.Add(this.textBox_IP2);
			this.panel1.Controls.Add(this.textBox_IP1);
			this.panel1.Controls.Add(this.label1);
			this.panel1.Controls.Add(this.txtbox_id);
			this.panel1.Controls.Add(this.label2);
			this.panel1.Controls.Add(this.btnAddNode);
			this.panel1.Controls.Add(this.NodelistView);
			this.panel1.Size = new System.Drawing.Size(240, 304);
			// 
			// Exit_Btn
			// 
			this.Exit_Btn.Location = new System.Drawing.Point(152, 72);
			this.Exit_Btn.Size = new System.Drawing.Size(72, 24);
			this.Exit_Btn.Text = "Exit";
			this.Exit_Btn.Click += new System.EventHandler(this.Exit_Btn_Click);
			// 
			// Set_VarBtn
			// 
			this.Set_VarBtn.Location = new System.Drawing.Point(72, 104);
			this.Set_VarBtn.Size = new System.Drawing.Size(152, 24);
			this.Set_VarBtn.Text = "Set Other Variables";
			this.Set_VarBtn.Click += new System.EventHandler(this.Set_VarBtn_Click);
			// 
			// ACM_CB
			// 
			this.ACM_CB.Checked = true;
			this.ACM_CB.CheckState = System.Windows.Forms.CheckState.Checked;
			this.ACM_CB.Location = new System.Drawing.Point(8, 136);
			this.ACM_CB.Size = new System.Drawing.Size(224, 24);
			this.ACM_CB.Text = "All nodes Connected with Multi-hops";
			this.ACM_CB.CheckStateChanged += new System.EventHandler(this.ACM_CB_CheckStateChanged);
			// 
			// btnReset
			// 
			this.btnReset.Location = new System.Drawing.Point(8, 104);
			this.btnReset.Size = new System.Drawing.Size(56, 24);
			this.btnReset.Text = "Reset";
			this.btnReset.Click += new System.EventHandler(this.btnReset_Click);
			// 
			// btnRmvNode
			// 
			this.btnRmvNode.Location = new System.Drawing.Point(72, 72);
			this.btnRmvNode.Size = new System.Drawing.Size(72, 24);
			this.btnRmvNode.Text = "Remove";
			this.btnRmvNode.Click += new System.EventHandler(this.btnRmvNode_Click);
			// 
			// textBox_IP4
			// 
			this.textBox_IP4.Location = new System.Drawing.Point(192, 40);
			this.textBox_IP4.Size = new System.Drawing.Size(40, 22);
			this.textBox_IP4.Text = "";
			// 
			// textBox_IP3
			// 
			this.textBox_IP3.Location = new System.Drawing.Point(144, 40);
			this.textBox_IP3.Size = new System.Drawing.Size(40, 22);
			this.textBox_IP3.Text = "";
			// 
			// textBox_IP2
			// 
			this.textBox_IP2.Location = new System.Drawing.Point(96, 40);
			this.textBox_IP2.Size = new System.Drawing.Size(40, 22);
			this.textBox_IP2.Text = "";
			// 
			// textBox_IP1
			// 
			this.textBox_IP1.Location = new System.Drawing.Point(48, 40);
			this.textBox_IP1.Size = new System.Drawing.Size(40, 22);
			this.textBox_IP1.Text = "";
			// 
			// btnGENConf
			// 
			this.btnGENConf.Location = new System.Drawing.Point(88, 248);
			this.btnGENConf.Size = new System.Drawing.Size(128, 24);
			this.btnGENConf.Text = "Generate conf.xml";
			this.btnGENConf.Click += new System.EventHandler(this.btnGENConf_Click);
			// 
			// panel_Variable
			// 
			this.panel_Variable.BackColor = System.Drawing.Color.GreenYellow;
			this.panel_Variable.Controls.Add(this.label9);
			this.panel_Variable.Controls.Add(this.Seed_TB);
			this.panel_Variable.Controls.Add(this.panel_Speed);
			this.panel_Variable.Controls.Add(this.panel_Pause_Time);
			this.panel_Variable.Controls.Add(this.Back_Btn);
			this.panel_Variable.Controls.Add(this.Power_Range_TB);
			this.panel_Variable.Controls.Add(this.label6);
			this.panel_Variable.Controls.Add(this.label5);
			this.panel_Variable.Controls.Add(this.label4);
			this.panel_Variable.Controls.Add(this.SA_Height_TB);
			this.panel_Variable.Controls.Add(this.SA_Width_TB);
			this.panel_Variable.Controls.Add(this.label3);
			this.panel_Variable.Controls.Add(this.btnGENConf);
			this.panel_Variable.Location = new System.Drawing.Point(0, 304);
			this.panel_Variable.Size = new System.Drawing.Size(240, 304);
			// 
			// label9
			// 
			this.label9.Location = new System.Drawing.Point(8, 232);
			this.label9.Size = new System.Drawing.Size(96, 16);
			this.label9.Text = "Enter seed :";
			// 
			// Seed_TB
			// 
			this.Seed_TB.Location = new System.Drawing.Point(120, 224);
			this.Seed_TB.Size = new System.Drawing.Size(72, 22);
			this.Seed_TB.Text = "3";
			// 
			// panel_Speed
			// 
			this.panel_Speed.BackColor = System.Drawing.Color.GreenYellow;
			this.panel_Speed.Controls.Add(this.Speed_80_RB);
			this.panel_Speed.Controls.Add(this.Speed_12_RB);
			this.panel_Speed.Controls.Add(this.Speed_10_RB);
			this.panel_Speed.Controls.Add(this.Speed_8_RB);
			this.panel_Speed.Controls.Add(this.Speed_6_RB);
			this.panel_Speed.Controls.Add(this.Speed_4_RB);
			this.panel_Speed.Controls.Add(this.Speed_2_RB);
			this.panel_Speed.Controls.Add(this.Speed_0_RB);
			this.panel_Speed.Controls.Add(this.label7);
			this.panel_Speed.Location = new System.Drawing.Point(0, 80);
			this.panel_Speed.Size = new System.Drawing.Size(240, 72);
			// 
			// Speed_80_RB
			// 
			this.Speed_80_RB.Checked = true;
			this.Speed_80_RB.Location = new System.Drawing.Point(168, 40);
			this.Speed_80_RB.Size = new System.Drawing.Size(56, 20);
			this.Speed_80_RB.Text = "80";
			// 
			// Speed_12_RB
			// 
			this.Speed_12_RB.Location = new System.Drawing.Point(112, 40);
			this.Speed_12_RB.Size = new System.Drawing.Size(40, 20);
			this.Speed_12_RB.Text = "12";
			// 
			// Speed_10_RB
			// 
			this.Speed_10_RB.Location = new System.Drawing.Point(64, 40);
			this.Speed_10_RB.Size = new System.Drawing.Size(40, 24);
			this.Speed_10_RB.Text = "10";
			// 
			// Speed_8_RB
			// 
			this.Speed_8_RB.Location = new System.Drawing.Point(16, 40);
			this.Speed_8_RB.Size = new System.Drawing.Size(32, 24);
			this.Speed_8_RB.Text = "8";
			// 
			// Speed_6_RB
			// 
			this.Speed_6_RB.Location = new System.Drawing.Point(184, 8);
			this.Speed_6_RB.Size = new System.Drawing.Size(40, 20);
			this.Speed_6_RB.Text = "6";
			// 
			// Speed_4_RB
			// 
			this.Speed_4_RB.Location = new System.Drawing.Point(144, 8);
			this.Speed_4_RB.Size = new System.Drawing.Size(32, 20);
			this.Speed_4_RB.Text = "4";
			// 
			// Speed_2_RB
			// 
			this.Speed_2_RB.Location = new System.Drawing.Point(104, 8);
			this.Speed_2_RB.Size = new System.Drawing.Size(40, 20);
			this.Speed_2_RB.Text = "2";
			// 
			// Speed_0_RB
			// 
			this.Speed_0_RB.Location = new System.Drawing.Point(64, 8);
			this.Speed_0_RB.Size = new System.Drawing.Size(32, 20);
			this.Speed_0_RB.Text = "0";
			// 
			// label7
			// 
			this.label7.Font = new System.Drawing.Font("Tahoma", 9F, System.Drawing.FontStyle.Bold);
			this.label7.Location = new System.Drawing.Point(8, 8);
			this.label7.Size = new System.Drawing.Size(48, 16);
			this.label7.Text = "Speed :";
			// 
			// panel_Pause_Time
			// 
			this.panel_Pause_Time.BackColor = System.Drawing.Color.GreenYellow;
			this.panel_Pause_Time.Controls.Add(this.PT_30_RB);
			this.panel_Pause_Time.Controls.Add(this.label8);
			this.panel_Pause_Time.Controls.Add(this.PT_15_RB);
			this.panel_Pause_Time.Controls.Add(this.PT_60_RB);
			this.panel_Pause_Time.Controls.Add(this.PT_120_RB);
			this.panel_Pause_Time.Controls.Add(this.PT_240_RB);
			this.panel_Pause_Time.Controls.Add(this.PT_480_RB);
			this.panel_Pause_Time.Controls.Add(this.PT_960_RB);
			this.panel_Pause_Time.Controls.Add(this.PT_0_RB);
			this.panel_Pause_Time.Location = new System.Drawing.Point(0, 152);
			this.panel_Pause_Time.Size = new System.Drawing.Size(240, 72);
			// 
			// PT_30_RB
			// 
			this.PT_30_RB.Location = new System.Drawing.Point(160, 8);
			this.PT_30_RB.Size = new System.Drawing.Size(40, 24);
			this.PT_30_RB.Text = "30";
			// 
			// label8
			// 
			this.label8.Font = new System.Drawing.Font("Tahoma", 9F, System.Drawing.FontStyle.Bold);
			this.label8.Location = new System.Drawing.Point(0, 8);
			this.label8.Size = new System.Drawing.Size(88, 24);
			this.label8.Text = "Pause Time :";
			// 
			// PT_15_RB
			// 
			this.PT_15_RB.Location = new System.Drawing.Point(120, 8);
			this.PT_15_RB.Size = new System.Drawing.Size(40, 24);
			this.PT_15_RB.Text = "15";
			// 
			// PT_60_RB
			// 
			this.PT_60_RB.Checked = true;
			this.PT_60_RB.Location = new System.Drawing.Point(200, 8);
			this.PT_60_RB.Size = new System.Drawing.Size(40, 24);
			this.PT_60_RB.Text = "60";
			// 
			// PT_120_RB
			// 
			this.PT_120_RB.Location = new System.Drawing.Point(24, 32);
			this.PT_120_RB.Size = new System.Drawing.Size(48, 24);
			this.PT_120_RB.Text = "120";
			// 
			// PT_240_RB
			// 
			this.PT_240_RB.Location = new System.Drawing.Point(80, 32);
			this.PT_240_RB.Size = new System.Drawing.Size(48, 24);
			this.PT_240_RB.Text = "240";
			// 
			// PT_480_RB
			// 
			this.PT_480_RB.Location = new System.Drawing.Point(128, 32);
			this.PT_480_RB.Size = new System.Drawing.Size(48, 24);
			this.PT_480_RB.Text = "480";
			// 
			// PT_960_RB
			// 
			this.PT_960_RB.Location = new System.Drawing.Point(184, 32);
			this.PT_960_RB.Size = new System.Drawing.Size(48, 24);
			this.PT_960_RB.Text = "960";
			// 
			// PT_0_RB
			// 
			this.PT_0_RB.Location = new System.Drawing.Point(88, 8);
			this.PT_0_RB.Size = new System.Drawing.Size(32, 24);
			this.PT_0_RB.Text = "0";
			// 
			// Back_Btn
			// 
			this.Back_Btn.Location = new System.Drawing.Point(8, 248);
			this.Back_Btn.Size = new System.Drawing.Size(72, 24);
			this.Back_Btn.Text = "Back";
			this.Back_Btn.Click += new System.EventHandler(this.Back_Btn_Click);
			// 
			// Power_Range_TB
			// 
			this.Power_Range_TB.Location = new System.Drawing.Point(120, 56);
			this.Power_Range_TB.Size = new System.Drawing.Size(96, 22);
			this.Power_Range_TB.Text = "100";
			// 
			// label6
			// 
			this.label6.Font = new System.Drawing.Font("Tahoma", 9F, System.Drawing.FontStyle.Bold);
			this.label6.Location = new System.Drawing.Point(8, 56);
			this.label6.Size = new System.Drawing.Size(104, 24);
			this.label6.Text = "Power Range :";
			// 
			// label5
			// 
			this.label5.Font = new System.Drawing.Font("Tahoma", 9F, System.Drawing.FontStyle.Bold);
			this.label5.Location = new System.Drawing.Point(112, 32);
			this.label5.Size = new System.Drawing.Size(56, 24);
			this.label5.Text = "Height :";
			this.label5.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// label4
			// 
			this.label4.Font = new System.Drawing.Font("Tahoma", 9F, System.Drawing.FontStyle.Bold);
			this.label4.Location = new System.Drawing.Point(0, 32);
			this.label4.Size = new System.Drawing.Size(56, 24);
			this.label4.Text = "Width :";
			this.label4.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// SA_Height_TB
			// 
			this.SA_Height_TB.Location = new System.Drawing.Point(176, 32);
			this.SA_Height_TB.Size = new System.Drawing.Size(40, 22);
			this.SA_Height_TB.Text = "200";
			// 
			// SA_Width_TB
			// 
			this.SA_Width_TB.Location = new System.Drawing.Point(64, 32);
			this.SA_Width_TB.Size = new System.Drawing.Size(40, 22);
			this.SA_Width_TB.Text = "200";
			// 
			// label3
			// 
			this.label3.Font = new System.Drawing.Font("Tahoma", 9F, System.Drawing.FontStyle.Bold);
			this.label3.Location = new System.Drawing.Point(8, 8);
			this.label3.Size = new System.Drawing.Size(112, 24);
			this.label3.Text = "Simulation Area :";
			// 
			// GENUI
			// 
			this.ClientSize = new System.Drawing.Size(240, 608);
			this.Controls.Add(this.panel_Variable);
			this.Controls.Add(this.panel1);
			this.Menu = this.mainMenu1;
			this.Text = "GENUI";
			this.Load += new System.EventHandler(this.GENUI_Load);

		}
		#endregion

		private void btnAddNode_Click(object sender, System.EventArgs e)
		{
			
			strID = txtbox_id.Text;
			strIP = textBox_IP1.Text+"."+textBox_IP2.Text+"."+
				textBox_IP3.Text+"."+textBox_IP4.Text;
			
			
			if(errorCheckForInputs())
			{
				id_ip_hash.Add(strID, strIP);
				//Show in listView
				showListView();
				txtbox_id.Text="";
				textBox_IP1.Text="";
				textBox_IP2.Text="";
				textBox_IP3.Text="";
				textBox_IP4.Text="";
			}			
		}

		private bool errorCheckForInputs()
		{
			if(id_ip_hash.ContainsKey(strID))
			{
				MessageBox.Show("The entered ID already exists.");
				return false;
			}
			else if(id_ip_hash.ContainsValue(strIP))
			{
				MessageBox.Show("The entered IP already exists.");
				return false;
			}
			else if(!isCorrectIP())
			{
				MessageBox.Show("The entered IP is not a correct form.");
				return false;                
			}
			else
			{
				return true;
			}
		}
		private bool isCorrectIP()
		{
			try
			{
				int firstThree = int.Parse(textBox_IP1.Text);
				int secondThree = Int32.Parse(textBox_IP2.Text);
				int ThirdThree = Int32.Parse(textBox_IP3.Text);
				int FourthThree = Int32.Parse(textBox_IP4.Text);
				if((firstThree>=0 && firstThree<=255)
					&& (secondThree>=0 && secondThree<=255)
					&& (ThirdThree>=0 && ThirdThree<=255)
					&& (FourthThree>=0 && FourthThree<=255))
				{
					return true;
				}
				return false;
			}
			catch(Exception e)
			{
				//	MessageBox.Show("The exception is occurred in correctIP "+e.Message);
				return false;
			}
		}//correct ip

		private void showListView()
		{
			try
			{
				NodelistView.Clear();
				NodelistView.View = View.Details;
				NodelistView.FullRowSelect = true;
				//			NodelistView.GridLines = false;
				int length = id_ip_hash.Count;
				ListViewItem[] view_id = new ListViewItem[length];
            
				IDictionaryEnumerator ide = id_ip_hash.GetEnumerator();
				int i=0;
				while(ide.MoveNext())
				{			
					view_id[i] = new ListViewItem(ide.Key.ToString());
					view_id[i].SubItems.Add(ide.Value.ToString());
					i++;
				}
				//create list view colums
				NodelistView.Columns.Add("ID", -2, HorizontalAlignment.Left);
				NodelistView.Columns.Add("IP Address            ",-2, HorizontalAlignment.Left);

				//Add items to list view
				for(int j=0; j<length; j++)
				{
					NodelistView.Items.Add(view_id[j]);
				}
			}
			catch(Exception e)
			{
               MessageBox.Show("Exception in showListView(): "+e.Message);
			}
		}

		private int getSpeed()
		{
			try
			{
				
				int speed = 0;
				if(Speed_0_RB.Checked)
				{
					speed = 0;
				}
				else if(Speed_2_RB.Checked)
				{
					speed = 2;
				}
				else if(Speed_4_RB.Checked)
				{
					speed = 4;
				}
				else if(Speed_6_RB.Checked)
				{
					speed = 6;
				}
				else if(Speed_8_RB.Checked)
				{
                    speed = 8;
				}
				else if(Speed_10_RB.Checked)
				{
					speed = 10;
				}
				else if(Speed_12_RB.Checked)
				{
					speed = 12;
				}
				else// 80
				{
					//speed = int.MaxValue;
					speed = 80;
				}
				return speed;
				/*int speed = int.Parse(Speed_TB.Text);
				return speed;*/
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in getSpeed(): "+e.Message);
				return -1;
			}
		}
		private int getPauseTime()
		{
			try
			{
				int PT = 0;
				if(PT_15_RB.Checked)
				{
				    PT = 15;
				}
				else if(PT_30_RB.Checked)
				{
					PT = 30;				
				}
				else if(PT_60_RB.Checked)
				{
					PT = 60;				
				}
				else if(PT_120_RB.Checked)
				{
					PT = 120;
				}
				else if(PT_240_RB.Checked)
				{
					PT = 240;
				}
				else if(PT_480_RB.Checked)
				{
					PT = 480;
				}
				else if(PT_960_RB.Checked)
				{
					PT = 960;
				}
				else //Random
				{
					//PT = int.MaxValue;
					PT = 0;
				}
				return PT;
	        }
			catch(Exception e)
			{
				MessageBox.Show("Exception in getPauseTime(): "+e.Message);
				return -1;
			}
		}
		private void SetVariables()
		{
			try
			{
				try
				{
					_SA_Width = int.Parse(SA_Width_TB.Text);
				}
				catch(Exception ex)
				{
					MessageBox.Show("Your input for simulation area's width is not valid!");
				}
				try
				{
					_SA_Height = int.Parse(SA_Height_TB.Text);
				}
				catch(Exception ex)
				{
					MessageBox.Show("Your input for simulation area's height is not valid!");
				}					
				try
				{
					_PowerRange = int.Parse(Power_Range_TB.Text);
				}
				catch(Exception ex)
				{
					MessageBox.Show("Your input for power range is not valid!");   
				}
				_Speed = getSpeed();
				_PauseTime = getPauseTime();
			}
			catch(Exception e)
			{
                MessageBox.Show("Exception in SetVariables(): "+e.Message);
			}
		}
		/// <summary>
		/// The conf.xml file is generated and the screen goes to the graph.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void btnGENConf_Click(object sender, System.EventArgs e)
		{
			try
			{
				if(NodelistView.Items.Count!=0)
				{
					SetACM();
					SetVariables();
					int numDes = GEN_CONSTANTS.NUM_OF_DESTINATION;
					generateConfXML(numDes);
					writeBasicInfoXML();

					//Go to the Network Topology Page
					goToDisplayNT();
				}
				else
				{
                    MessageBox.Show("Please add nodes in the list.");
				}
			}
			catch(Exception ex)
			{
                MessageBox.Show("Exception in btnGENConf_Click(): "+ex.Message+xmlDoc.ToString());
			}
		}


		private void generateConfXML(int p_numDes)
		{
			try
			{					       
				all_sequences = new ArrayList();
				int numNodes = id_ip_hash.Count;
				int numDes = p_numDes;
				GENConf confXML = new GENConf(numNodes,
					                          numDes,
					                          _SA_Width,
					                          _SA_Height,
					                          _Speed,
					                          _PauseTime);
				try
				{
					int seed =	int.Parse(Seed_TB.Text);          
					int[] seeds = confXML.generateSeeds(seed);
					for(int i=0; i<seeds.Length; i++)
					{
						confXML.generateSequenceForOneNode(seeds[i]);
						Sequence[] rl = confXML.getSequenceList();
						Sequence[] temp = new Sequence[numDes];
						rl.CopyTo(temp,0);
						all_sequences.Add(temp);
					}
				}
				catch(FormatException fex)
				{
                    MessageBox.Show("Seed format is not valid!");
				}
				writeConfXML();	
			}
			catch(Exception e)
			{
                MessageBox.Show("Exception in generateConfXML(): "+e.Message);
			}
		}
		private void writeBasicInfoXML()
		{
			try
			{
				basicInfoWriter = 
					new XmlTextWriter(GEN_CONSTANTS.FILE_NAME_BASICINFO, Encoding.ASCII);
				basicInfoWriter.Formatting = Formatting.Indented;
				basicInfoWriter.WriteStartDocument();
				basicInfoWriter.WriteStartElement("GEN_BasicInfo");//root <GEN_BasicInfo>
				int temp_numOfNodes = id_ip_hash.Count;
				basicInfoWriter.WriteElementString("NumOfNodes",temp_numOfNodes.ToString());//<NumOfNodes></NumOfNodes>
				basicInfoWriter.WriteStartElement("SimulationArea");//<SimulationArea>
				basicInfoWriter.WriteElementString("Width",_SA_Width.ToString());
				basicInfoWriter.WriteElementString("Height",_SA_Height.ToString());
				basicInfoWriter.WriteEndElement();//</SimulationArea>
				basicInfoWriter.WriteElementString("PowerRange",_PowerRange.ToString());
				string strSpeed = "";
				if(_Speed == int.MaxValue)
				{
					strSpeed = "Random";
				}
				else
				{
					strSpeed = _Speed.ToString();				
				}
				string strPauseTime = "";
				if(_PauseTime == int.MaxValue)
				{
                   strPauseTime = "Random";
				}
				else
				{
                   strPauseTime = _PauseTime.ToString();
				}
				basicInfoWriter.WriteElementString("Speed",strSpeed);
		        basicInfoWriter.WriteElementString("PauseTime",strPauseTime);
				basicInfoWriter.WriteEndElement();//</GEN_BasicInfo>
				basicInfoWriter.WriteEndDocument();
				basicInfoWriter.Close();
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in writeBasicInfoXML(): "+e.Message);
			}
		}

		private void writeConfXML()
		{
			try
			{
				confWriter = 
						new XmlTextWriter(GEN_CONSTANTS.FILE_NAME_CONF,Encoding.ASCII);
			
				confWriter.Formatting = Formatting.Indented;
				confWriter.WriteStartDocument();
				confWriter.WriteStartElement("Configuration");
				IDictionaryEnumerator ide = id_ip_hash.GetEnumerator();
				int i=0;
				while(ide.MoveNext())
				{
					confWriter.WriteStartElement("Node");//<Node>
					confWriter.WriteAttributeString("ID",ide.Key.ToString());
					confWriter.WriteAttributeString("IP", ide.Value.ToString());
					Point startPoint = new Point();
					startPoint = getRandomStartPoint(i);
					string strSP ="("+startPoint.row.ToString()+","+startPoint.col.ToString()+")";
					confWriter.WriteAttributeString("StartPt",strSP);
					confWriter.WriteStartElement("Sequence");//<Sequence>
					//Store a sequence
					Sequence[] seq = (Sequence[])all_sequences[i];
					i++;
					for(int j=0; j<seq.Length; j++)
					{
						confWriter.WriteStartElement("Hop");//<Hop>
						Point dest = seq[j].destination;
						int speed = seq[j].speed;
						int pause = seq[j].pause;
						confWriter.WriteElementString("Destination","("+dest.row+","+dest.col+")");
						confWriter.WriteElementString("Speed",speed.ToString());
						confWriter.WriteElementString("Pause",pause.ToString());
						confWriter.WriteEndElement();//</Hop>
					}//for
                     
					confWriter.WriteEndElement();//</Sequence>
					confWriter.WriteEndElement();//</Node>
				}//while
				confWriter.WriteEndElement();//</configuration>
				confWriter.WriteEndDocument();
				//confWriter.Flush();
				confWriter.Close();
			}//try
			catch(Exception e)
			{
				MessageBox.Show("Exception is occurred in writeConfXML "+e.Message);
			}
		}//writeConfXML

		private Point getRandomStartPoint(int index)
		{
			Random r_row;
			Random r_col;

			if(index%2==0)
			{
				r_row = new Random(index+50);
				r_col = new Random(index+20);
			}
			else
			{
				r_row = new Random(index+20);
				r_col = new Random(index+50);
			}
			int row = r_row.Next(0, 200);
			int col = r_col.Next(0, 200);
			Point startPoint = new Point(Convert.ToDouble(row),Convert.ToDouble(col));
			return startPoint;
		}
		
		/// <summary>
		/// The selected node is removed from the list view.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void btnRmvNode_Click(object sender, System.EventArgs e)
		{
			string rmv_node_key = NodelistView.Items
				[(NodelistView.SelectedIndices[0])].Text;
			id_ip_hash.Remove(rmv_node_key);
			
			showListView(); 
		}

		private void resetAllRB()
		{
			Speed_0_RB.Checked = false;
			Speed_2_RB.Checked = false;
			Speed_4_RB.Checked = false;
			Speed_6_RB.Checked = false;
			Speed_8_RB.Checked = false;
			Speed_10_RB.Checked = false;
			Speed_12_RB.Checked = false;
			Speed_80_RB.Checked = true;			
			PT_15_RB.Checked = false;
			PT_30_RB.Checked = false;
			PT_60_RB.Checked = false;
			PT_120_RB.Checked = false;
			PT_240_RB.Checked = false;
			PT_480_RB.Checked = false;
			PT_960_RB.Checked = false;
			PT_0_RB.Checked = true;
		}
		private void btnReset_Click(object sender, System.EventArgs e)
		{
			id_ip_hash.Clear();
			ACM_CB.Checked = false;
			resetAllRB();
			SA_Width_TB.Text = "200";
			SA_Height_TB.Text = "200";
			Power_Range_TB.Text = "40";
			showListView();
		}
		private bool isSamePoint(Point p1, Point p2)
		{
			if(((p1.row==p2.row) && (p1.col==p2.col)))
			{
				return true;               
			}
			return false;
		}
		
		private void SetACM()
		{
			try
			{
				if(ACM_CB.Checked)
				{
					_isOnACM = true;
				}
				else
				{
					_isOnACM = false;
				}
			}
			catch(Exception e)
			{
				MessageBox.Show("Exception in SetACM(): "+e.Message); 
			}
		}

		private void ACM_CB_CheckStateChanged(object sender, System.EventArgs e)
		{
			try
			{
				SetACM();
			}
			catch(Exception ex)
			{
				MessageBox.Show("Exception in ACM_CB_CheckStateChanged(): "+ex.Message); 
			}
		
		}

		private void goToDisplayNT()
		{
			GENUI_NT spec = new GENUI_NT(_isOnACM, 
				                         id_ip_hash, 
				                         _PowerRange,
				                         _SA_Width,
				                         _SA_Height);
			spec.Top = 0;
			spec.Visible = true;
			spec.Focus();
			spec.BringToFront();
		}


		private void Set_VarBtn_Click(object sender, System.EventArgs e)
		{
			if(id_ip_hash.Count!=0)
			{
				panel_Variable.Top = 0;
				panel_Variable.Focus();
				panel_Variable.BringToFront();
				panel_Variable.Visible = true;
			}
			else
			{
               MessageBox.Show("Please enter the participating nodes first!");
			}
		}

		private void Back_Btn_Click(object sender, System.EventArgs e)
		{
			panel1.Top = 0;
			panel1.Focus();
			panel1.BringToFront();
			panel1.Visible = true;
		}

		private void GENUI_Load(object sender, System.EventArgs e)
		{
			showListView();
		}

		private void Exit_Btn_Click(object sender, System.EventArgs e)
		{
			this.Close();
		}

		
	}//class	
}
